#clear
rm(list=ls())

#function for roc curve
roc_curve <- function(y,score){
  #number of examples
  n <- length(y)
  #number of positive examples
  pos <- sum(y)
  #number of negative examples
  neg <- n - pos
  #size of the target
  target <- seq(1,n,1)
  #sorting values
  index <- sort(score,decreasing=T,index.return=T)
  sy <- y[index$ix]
  sscore <- score[index$ix]
  #cumulative number of positives
  c.pos <- cumsum(sy)
  #TPR - true positive rate
  tpr <- c.pos/pos
  tpr <- c(0,tpr)
  #cumulative number of negatives
  c.neg <- target - c.pos
  #FPR - false positive rate
  fpr <- c.neg/neg
  fpr <- c(0,fpr)
  #return values
  return(list(x=fpr,y=tpr))
}

#function for cumulative lift curve
cum_lift_curve <- function(y,score){
  #number of examples
  n <- length(y)
  #number of positive examples
  pos <- sum(y)
  #size of the target
  target <- seq(1,n,1)
  #sorting values
  index <- sort(score,decreasing=T,index.return=T)
  sy <- y[index$ix]
  sscore <- score[index$ix]
  #cumulative number of positives
  c.pos <- cumsum(sy)
  #TPR - true positive rate
  tpr <- c.pos/pos
  tpr <- c(0,tpr)
  #relative size of the target
  rel.target <- target/n
  rel.target <- c(0,rel.target)
  #return values
  return(list(x=rel.target,y=tpr))
}

#function for recall-precision curve
recall_prec <- function(y,score){
  #number of examples
  n <- length(y)
  #number of positive examples
  pos <- sum(y)
  #size of the target
  target <- seq(1,n,1)
  #sorting values
  index <- sort(score,decreasing=T,index.return=T)
  sy <- y[index$ix]
  sscore <- score[index$ix]
  #cumulative number of positives
  c.pos <- cumsum(sy)
  #TPR - true positive rate
  tpr <- c.pos/pos
  #precision
  prec <- c.pos/target
  #return values
  return(list(x=tpr,y=prec))
}

#library for xls
library(xlsReadWrite)
#load the dataset
all.data <- read.xls(file="D://DataMining//Databases_for_mining//dataset_for_soft_dev_and_comparison//curves_for_spv_learning//heart_disease_for_curves.xls",sheet=1)
#train and test set
train.data <- all.data[all.data$sample=="train",2:5]
print(summary(train.data))
test.data <- all.data[all.data$sample=="test",2:5]
print(summary(test.data))
#y as 0/1 on the test set
y <- ifelse(test.data$disease=="positive",1,0)
#logistic regression
lr.model <- glm(disease ~ ., data = train.data, family = "binomial")
print(lr.model)
#score on the test set
lr.score <- predict(lr.model, newdata = test.data, type="response")
print(summary(lr.score))
#svm
library(e1071)
svm.model <- svm(disease ~ ., data=train.data, probability=T, kernel = "radial")
print(svm.model)
#score on the test set
svm.pred <- predict(svm.model, newdata = test.data, probability=T)
svm.score <- attr(svm.pred,"probabilities")[,1]
print(summary(svm.score))

#ROC curve
lr.roc <- roc_curve(y,lr.score)
svm.roc <- roc_curve(y,svm.score)
plot(lr.roc$x,lr.roc$y,type="l",col="blue",main="ROC Curve",xlab="FPR",ylab="TPR")
lines(svm.roc$x,svm.roc$y,type="l",col="red")
abline(0,1)

#Cumulative lift curve
lr.lift <- cum_lift_curve(y,lr.score)
svm.lift <- cum_lift_curve(y,svm.score)
plot(lr.lift$x,lr.lift$y,type="l",col="blue",main="Gain chart",xlab="#cible/n",ylab="TPR")
lines(svm.lift$x,svm.lift$y,type="l",col="red")
abline(0,1)

#Recall precision curve
lr.rp <- recall_prec(y,lr.score)
svm.rp <- recall_prec(y,svm.score)
plot(lr.rp$x,lr.rp$y,type="l",col="blue",main="Precision-Recall",xlab="Recall",ylab="Precision")
lines(svm.rp$x,svm.rp$y,type="l",col="red")
